from .thing import Thing


class Wall(Thing):
    height = 5
    length = 10
    width = 2

    def build(self):
        self.server.setBlocks(
            self.position.x, self.position.y, self.position.z,
            self.position.x+self.length-1,
            self.position.y+self.height-1,
            self.position.z+self.width-1,
            self.block)



from .thing import Thing


class Block(Thing):

    def build(self):
        self.server.setBlock(self.position.x, self.position.y,
                             self.position.z, self.block)

        self._end_position = self.position



import mcpi

from .thing import Thing
from .house import House


class Building(Thing):

    floors = 10
    width = 10

    def build(self):
        init_x = self.position.x
        init_y = self.position.y
        init_z = self.position.z

        house_pos = mcpi.vec3.Vec3(init_x, init_y, init_z)
        init_height = init_y

        for i in range(0, self.floors):
            house = House(self.server, house_pos)
            house_pos.y = house.height * i + init_height
            house.width = self.width
            house.block = self.block
            house.build()
            self._end_position = house.end_position



from mcpi.vec3 import Vec3


from .thing import Thing


class Pyramid(Thing):

    height = 10

    def build(self):
        length = 2 * self.height - 1
        width = length

        for i in range(0, self.height):
            level = i
            self.server.setBlocks(
                self.position.x + level, self.position.y + level, self.position.z + level,
                self.position.x + (length - 1) - level,
                self.position.y + level,
                self.position.z + (width - 1) - level,
                self.block)

        self._end_position = Vec3(self.position.x + (length - 1),
                                  self.position.y + self.height,
                                  self.position.z + (width - 1)
                                  )



import mcpi

from .thing import Thing
from .wall import Wall


class Fence(Thing):
    """
    Build a block plane and empty it to create the fence
    """

    height = 3
    fence_space = 5
    """Space between the fence and the thing fenced"""
    thick = 1
    thing = None

    def build(self):

        if self.thing is None:
            raise RuntimeError("Thing to be fenced is not defined")

        self._position = self.thing.position
        self._end_position = self.thing.end_position

        init_x = self.thing.position.x - self.fence_space - self.thick
        init_y = self.thing.position.y
        init_z = self.thing.position.z - self.fence_space - self.thick

        end_x = self.thing.end_position.x + self.fence_space + self.thick
        end_y = self.thing.end_position.y
        end_z = self.thing.end_position.z + self.fence_space + self.thick

        self.server.setBlocks(
            init_x, init_y, init_z,
            end_x, end_y, end_z,
            self.block)

        # Fill the prism with air to became a rectangular wall
        self.server.setBlocks(
            init_x + self.thick, init_y, init_z + self.thick,
            end_x - self.thick,
            end_y,
            end_z - self.thick,
            mcpi.block.AIR)

        # Rebuild the thing because it is destroyed when emptying the fence
        self.thing.build()



import mcpi

from mcpi.vec3 import Vec3

from .thing import Thing


class House(Thing):

    height = 3
    width = 5
    length = 5
    wall_width = 1
    door_size = 1

    def build(self):
        init_x = self.position.x
        init_y = self.position.y
        init_z = self.position.z

        end_x = init_x + self.length-1
        end_y = init_y + self.height-1
        end_z = init_z + self.width-1

        self.server.setBlocks(
            init_x, init_y, init_z,
            end_x, end_y, end_z,
            self.block)

        # Fill the cube with air so it becomes a kind of house
        self.server.setBlocks(
            init_x + self.wall_width, init_y, init_z + self.wall_width,
            end_x - self.wall_width,
            end_y - self.wall_width,
            end_z - self.wall_width,
            mcpi.block.AIR)

        # Add a door
        self.server.setBlocks(
            init_x, init_y, init_z + self.wall_width,
            init_x + 1, init_y + self.door_size, init_z + self.door_size,
            mcpi.block.AIR)

        self._end_position = Vec3(end_x, end_y, end_z)





import mcpi

from mcpi.vec3 import Vec3

from .thing import Thing


class BlocksGallery(Thing):

    # https://www.minecraftinfo.com/idlist.htm
    MAX_BLOCK_NUMBER = 247

    def build(self):
        """
        Show all possible block types in a line
        :return:
        """

        for i in range(1, self.MAX_BLOCK_NUMBER):
            self.server.setBlock(self.position.x + i, self.position.y,
                                 self.position.z, mcpi.block.Block(i))

        self._end_position = Vec3(self.position.x + self.MAX_BLOCK_NUMBER - 1,
                                  self.position.y, self.position.z)



import mcpi.block
import mcpi.vec3


class Thing:
    """ base class for all objects in mcthings library """

    block = mcpi.block.BRICK_BLOCK
    """ block type used by the thing. Default to BRICK_BLOCK"""
    _block_empty = mcpi.block.AIR

    @property
    def position(self):
        """ initial position of the thing """
        return self._position

    @property
    def end_position(self):
        """ end position of the thing """
        return self._end_position

    @property
    def server(self):
        """ Minecraft Python server connection """
        return self._server

    def __init__(self, server=None, position=None):
        """
        Create a thing
        :param server: Minecraft Python server in format host:port
        :param position: build position
        """

        self._end_position = None
        self._position = None
        if position:
            self._position = mcpi.vec3.Vec3(position.x, position.y, position.z)
        self._server = server

    def build(self):
        """
        Build the thing and show it in Minecraft at position coordinates

        :return:
        """
        pass

    def unbuild(self):
        """
        Unbuild the thing in Minecraft

        :return:
        """

        block = self.block
        self.block = self._block_empty
        self.build()
        self.block = block

    def move(self, position):
        """
        Move the thing to a new position

        :param position: new position
        :return:
        """

        self.unbuild()
        self._position = position
        self.build()



import mcpi

from .thing import Thing
from .house import House


class Town(Thing):

    houses = 4
    house_width = 5
    house_length = 5
    house_height = 3
    space = 3
    """space between the town houses"""

    def build(self):
        init_x = self.position.x
        init_y = self.position.y
        init_z = self.position.z

        house_pos = mcpi.vec3.Vec3(init_x, init_y, init_z)

        for i in range(0, self.houses):
            house = House(self.server, house_pos)
            house.width = self.house_width
            house.length = self.house_length
            house.height = self.house_height
            house.block = self.block
            house.build()
            house_pos.z += self.house_width + self.space

        # Fill the end_position
        end_x = init_x + self.house_length
        end_y = init_y + self.house_height
        end_z = house_pos.z - self.space

        self._end_position = mcpi.vec3.Vec3(end_x, end_y, end_z)
